#! /usr/bin/ksh
##################################################################################
# Copyright (c) 2007- IBM CORPORATION                                            #
#                                                                                #
# Rekey script to replace the encryption key in tape cartridge on UNIX platforms #
#  (AIX, Linux, Solaris and HP-UX)                                               #
# Usage:                                                                         #
#    rekey_unix.sh drive_dsf changer_dsf 1st_slot_addr last_slot_addr            #
#    for example on AIX run this command line:                                   #
#       rekey_unix.sh  rmt2.1 smc1 1027 2050                                     #
# Note: Due to the limited scope of testing, the code is distributed "AS IS".    #                                                                               #
##################################################################################

typeset -i  slot slot1 slot2 line1 line2

help ()
{
  cat <<end_help
  rekey
    - a sample shell script is used to replace the encryption key for tape cartridge
    located at the storage slot only on UNIX platforms(AIX, Linux, Solaris and HP-UX)

  Usage:
    rekey_unix.sh drive_dsf changer_dsf 1st_slot_addr last_slot_addr
    for example, enter this command line:
       rekey_unix.sh rmt2.1 smc1 1027 2050   on AIX
       rekey_unix.sh 0stn 0smc 1027 2050     on Solaris
       rekey_unix.sh 0mn  1chng 1027 2050    on HP-UX
       rekey_unix.sh IBMtape0n IBMchanger0 1027 2050 on Linux

      a)drive_dsf: drive device special file, such as the file with no rewind on close:
         AIX: rmt0.1  Solaris: 0stn  HP-UX: 0mn  Linux: IBMtape0n
      b)changer_dsf: changer device special file, for instance,
         AIX: smc0  Solaris: 0smc  HP-UX: 0chng  Linux: IBMchanger0
      c)start_slot_addr: the first slot address will be started for rekeying
      d)last_slot_addr: the last slot address will be stopped for rekeying

  Note:
    1. The utility supports for 3592 in Library Managed Encryption (LME) mode only.
    2. The script runs with IBM tape drivers only
    3. The changer must be configured with IBM tape driver
    4. The drive must be in the same logical library
    5. The rekey tape cartridges must be located in storage slots
    6. Make sure the tape isn't loaded in the drive before to run the scriptS
    7. Run the utility with the latest tape drive and library codes

end_help
}

########################
# check arguments
########################
if [ $# -lt 4 ] ; then
   help
   exit
fi

########################
# check operate system
########################
OS=$(uname -s)      # operating system name
if [ "$OS" = "AIX" ]; then
   OS_VERSION="$(uname -v)"
   drive=/dev/$1              # drive device special file (dsf)
   changer=/dev/$2            # changer device special file (dsf)
elif [ "$OS" = "Linux" ]; then
   OS_VERSION="$(uname -m)"
   ARCH="$(uname -i)"
   if [ -f /etc/asianux-release ] ; then
      DISTNAME=Asianux;
      DISTVER=`/bin/gawk '{print $3}' /etc/asianux-release`;
      DIST=${DISTNAME}${DISTVER};
   elif [ -f /etc/redhat-release ] ; then
      DISTNAME=RHEL;
      DISTVER=`/bin/gawk '{print $7}' /etc/redhat-release`;
      DIST=${DISTNAME}${DISTVER};
   elif [ -f /etc/SuSE-release ] ; then
      DISTNAME=SLES;
      DISTVER=`/bin/gawk '{print $5}' /etc/SuSE-release`;
      DIST=${DISTNAME}${DISTVER};
   fi

   drive=/dev/$1
   changer=/dev/$2
else                # Solaris and HP-UX
   OS_VERSION="$(uname -r)"
   drive=/dev/rmt/$1
   changer=/dev/rmt/$2
fi

slot1=$3             # start the rekey from the first slot address
slot2=$4             # stop the rekey at the last slot address

if [ $slot1 -gt $slot2 ] ; then
   echo "  Invalid last slot address !!!"
   echo ""
   help
   exit
fi

logtime="$(date '+20%y-%m-%d-%Hh-%Mm-%Ss')"
date                                                             > ./$OS/rekey_$1.$logtime.log
echo "rekey for encryption tape running on $OS $OS_VERSION"
echo "rekey for encryption tape running on $OS $OS_VERSION"      >> ./$OS/rekey_$1.$logtime.log
echo ""                                                          >> ./$OS/rekey_$1.$logtime.log

slot=$slot1

if [ "$OS" = "Linux"  ] ; then
   cp -f ./$OS/$DIST/IBMtapeutil.$ARCH ./$OS/tapeutil            >> ./$OS/rekey_$1.$logtime.log
   if [ -f ./$OS/tapeutil ] ; then
      chmod 555 ./$OS/tapeutil
   else
      echo "No find IBMtapeutil. Unsupport for $OS $OS_VERSION"
      echo "No find IBMtapeutil. Unsupport for $OS $OS_VERSION"  >> ./$OS/rekey_$1.$logtime.log
      echo "Please take a look $OS/rekey_$1.$logtime.log to find any error report in ths file!!"
      exit
   fi
fi

# check the encryption method
./$OS/tapeutil -f $drive encryption | grep Method | awk -F: '{print $1 $2 $3}' | read w1 w2 enc_method

if [ "$enc_method" != "Library" ] ; then
   echo "Unsupport the encryption managed method -- $enc_method !!"
   echo "Unsupport the encryption managed method -- $enc_method !!" >> ./$OS/rekey_$1.$logtime.log
   if [ "$OS" = "Linux"  ] ; then
      if [ -f ./$OS/tapeutil ] ; then
         rm -f ./$OS/tapeutil
      fi
   fi
   echo "Please take a look $OS/rekey_$1.$logtime.log to find any error report in ths file!!"
   exit
fi

# Get invertory and devid info
if [ "$OS" = "HP-UX" ] ; then
   ./$OS/tapeutil -f $changer -o inv -t 1 > ./$OS/inv_$2.out
   ./$OS/tapeutil -f $changer -o dvc -t 1 > ./$OS/devid_$2.out
else
   ./$OS/tapeutil -f $changer inventory > ./$OS/inv_$2.out
   ./$OS/tapeutil -f $changer devids > ./$OS/devid_$2.out
fi

# get drive sn and its address, and check if the drive exists in the library
sernum=$(./$OS/tapeutil -f $drive sernum)
echo "Get the sn $sernum of the drive $1"                    >> ./$OS/rekey_$1.$logtime.log

if [ "$OS" = "AIX" ] || [ "$OS" = "HP-UX" ] || [ "$OS" = "Linux" ] ; then
   grep -i -n $sernum ./$OS/devid_$2.out | awk -F: '{print $1 $2 $3 $4 $5 $6 $7}' | read line1 w2 w3 w4 w5 dev_model devid
elif [ "$OS" = "SunOS" ] ; then
   grep -i -n $sernum ./$OS/devid_$2.out | awk -F: '{print $1 $2 $3 $4 $5}' | read line1 w2 w3 w4 devid
fi

if [ "${sernum}" = "${devid}" ] ; then
   let line2=line1-10
   line=$(grep -i -n $line1 ./$OS/devid_$2.out | head -$line2 ./$OS/devid_$2.out | tail -8 | grep "Drive Address")
   echo $line | awk -F: '{print $1 $2 $3}' | read w1 w2 drive_addr
   echo "Found the drive $1 with devid $devid at drive address $drive_addr"  >> ./$OS/rekey_$1.$logtime.log
else
   echo "Not found the drive $1 in the library $2 !"
   echo "Not found the drive $1 in the library $2 !"    >> ./$OS/rekey_$1.$logtime.log
   rm -f ./$OS/inv_$2.out ./$OS/devid_$2.out
   if [ "$OS" = "Linux"  ] ; then
      if [ -f ./$OS/tapeutil ] ; then
         rm -f ./$OS/tapeutil
      fi
   fi
   echo "Please take a look $OS/rekey_$1.$logtime.log to find any error report in ths file!!"
   exit
fi

if [ "$OS" = "AIX" ]; then
   # unload and move the tape out before to rekey
   echo ""                                                      >> ./$OS/rekey_$1.$logtime.log
   echo "Ignore the error for this command, we just make sure no tape in the drive"   >> ./$OS/rekey_$1.$logtime.log
   ./$OS/tapeutil -f $changer unmount   2>/dev/null             >> ./$OS/rekey_$1.$logtime.log
fi

while [ $slot -le $slot2 ] ; do
    #  check if the slot is full and get the barcode
    if [ "$OS" = "AIX" ] || [ "$OS" = "Linux" ] ; then
       grep -i -n "Slot Address $slot" ./$OS/inv_$2.out | awk -F: '{print $1}' | read line1
    elif [ "$OS" = "SunOS" ] || [ "$OS" = "HP-UX" ] ; then
       grep -i -n "Storage Cell Address $slot" ./$OS/inv_$2.out | awk -F: '{print $1}' | read line1
    fi

    let line2=line1+7
    line=$(grep -i -n $line1 ./$OS/inv_$2.out | head -$line2 ./$OS/inv_$2.out | tail -8 | grep "Media Present")
    echo $line | awk -F: '{print $1 $2 $3 $4}' | read w1 w2 w3 slot_status
    line=$(grep -i -n $line1 ./$OS/inv_$2.out | head -$line2 ./$OS/inv_$2.out | tail -8 | grep "Volume Tag")
    echo $line | awk -F: '{print $1 $2 $3 $4}' | read w1 w2 w3 barcode

    # start to rekey
    if [ "$slot_status" = "Yes" ] ; then
       echo ""
       echo ""                                                   >> ./$OS/rekey_$1.$logtime.log
       echo "move tape $barcode from slot $slot to drive $1"
       echo "move tape $barcode from slot $slot to drive $1"     >> ./$OS/rekey_$1.$logtime.log
       if [ "$OS" = "HP-UX" ]; then
          ./$OS/tapeutil -f $changer -o mov -s $slot -d $drive_addr >> ./$OS/rekey_$1.$logtime.log
       else
          ./$OS/tapeutil -f $changer move $slot $drive_addr      >> ./$OS/rekey_$1.$logtime.log
       fi

       echo "rekey for the tape $barcode "
       echo "rekey for the tape $barcode "                       >> ./$OS/rekey_$1.$logtime.log
       if [ "$OS" = "HP-UX" ]; then
           ./$OS/tapeutil -f $drive rekey                         >> ./$OS/rekey_$1.$logtime.log
           ./$OS/tapeutil -f $drive -o lod -x 2                   >> ./$OS/rekey_$1.$logtime.log
       else
          if [ "$OS" = "Linux" ] ; then
             # sleep to wait for TUR ready
             sleep 30
          fi
          ./$OS/tapeutil -f $drive rekey unload                  >> ./$OS/rekey_$1.$logtime.log
       fi

       echo "move tape $barcode back to slot $slot"
       echo "move tape $barcode back to slot $slot"              >> ./$OS/rekey_$1.$logtime.log
       if [ "$OS" = "HP-UX" ]; then
          ./$OS/tapeutil -f $changer -o mov -s $drive_addr -d $slot >> ./$OS/rekey_$1.$logtime.log
       else
          ./$OS/tapeutil -f $changer move $drive_addr $slot         >> ./$OS/rekey_$1.$logtime.log
       fi
    fi

    let slot+=1
done

rm -f ./$OS/inv_$2.out ./$OS/devid_$2.out
if [ "$OS" = "Linux"  ] ; then
   if [ -f ./$OS/tapeutil ] ; then
      rm -f ./$OS/tapeutil
   fi
fi

echo ""
echo ""                                                          >> ./$OS/rekey_$1.$logtime.log
echo "End of the rekey!"                                         >> ./$OS/rekey_$1.$logtime.log
echo "End of the rekey!"
echo "Please take a look $OS/rekey_$1.$logtime.log to find any error report in ths file!!"


